"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Instrumenter = void 0;

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

class Instrumenter {
  constructor(contract) {
    this.contract = contract;
    this.injectionPoints = [];
  }

  getInjectionPoints() {
    var _context;

    return (0, _sort.default)(_context = this.injectionPoints).call(_context, (a, b) => {
      return a.node.loc && b.node.loc ? b.node.loc.start.line - a.node.loc.start.line : 0;
    });
  }

  instrumentContractDefinition(node) {
    if (!node.loc) {
      return;
    }

    if (node.loc.start.line === node.loc.end.line) {
      return;
    }

    this.addInjectionPoints("contractDefinition", 1, node);
  }

  instrumentFunction(node) {
    // Remove stateMutability so that the function is not exported
    // as "view" or "pure", seen as it will contain some emit
    // statements and those require a transaction.
    if (node.stateMutability === "view" || node.stateMutability === "pure") {
      node.stateMutability = undefined;
    }

    if (!node.loc) {
      return;
    }

    if (!node.body || !node.body.loc) {
      return;
    }

    if (node.body.loc.start.line === node.body.loc.end.line) {
      return;
    }

    this.contract.addFunction(node.loc, node.name || "", node.body.loc);
  }

  instrumentStatement(node, parent) {
    if (!node.loc) {
      return;
    }

    const id = this.contract.addStatement(node.loc);
    this.addInjectionPoints("statement", id, node, parent);
  }

  instrumentIfStatement(node) {
    var _context2;

    if (!node.loc) {
      return;
    } // Ensure that the trueBody and falseBody are Blocks. This prevents single
    // statement `if` checks from breaking when we try to add an emit statement.


    node.trueBody = this.wrapInBlock(node.trueBody);

    if (node.falseBody) {
      node.falseBody = this.wrapInBlock(node.falseBody);
    }

    const locations = (0, _reduce.default)(_context2 = [node.trueBody, node.falseBody]).call(_context2, (acc, body) => {
      if (body && body.loc) {
        acc.push(body.loc);
      }

      return acc;
    }, []);
    this.contract.addBranch(node.loc.start.line, "if", locations);
  }

  wrapInBlock(node) {
    if (node.type === "Block") {
      return node;
    }

    const newNode = {
      loc: node.loc,
      range: node.range,
      statements: [node],
      type: "Block"
    };
    return newNode;
  }

  addInjectionPoints(type, id, node, parent) {
    this.injectionPoints.push({
      type,
      id,
      node,
      parent
    });
  }

}

exports.Instrumenter = Instrumenter;
//# sourceMappingURL=instrumenter.js.map